;; Generated package description from load-relative.el  -*- no-byte-compile: t -*-
(define-package "load-relative" "1.3.1.0.20201201.144056" "Relative file load (within a multi-file Emacs package)" 'nil :keywords '("internal") :authors '(("Rocky Bernstein" . "rocky@gnu.org")) :maintainer '("Rocky Bernstein" . "rocky@gnu.org") :url "http://github.com/rocky/emacs-load-relative")
